/**
 * The simple NIM game.
 * There are 21 sticks in the pile. On each move, each user take turns picking
 * up 1, 2, or 3 sticks until there are no more sticks left.
 * The one who picks up the last stick loses.
 * @author Namita Singla, Robert Cohen
 */

public class SimpleNIM {

  private SimpleNIMGUI simpleNIMGUI;
  private int sticksLeft = 21;
  private Player humanPlayer;
  private Player machinePlayer;
  private Player turn; // current player
  private String whoGoFirst;

  /**
   * The constructor.
   * @param simpleNIMGUI The GUI associated with the game
   */
  public SimpleNIM(SimpleNIMGUI simpleNIMGUI) {
    this.simpleNIMGUI = simpleNIMGUI;
    setWhoGoFirst("Machine");
  }

  /**
   * Starts the game
   */
  public void start() {

    humanPlayer = new HumanPlayer(this);
    machinePlayer = new MachinePlayer(this);
    sticksLeft = 21;
    humanPlayer.start();
    machinePlayer.start();
    humanPlayer.setNext(machinePlayer);
    machinePlayer.setNext(humanPlayer);
    if (whoGoFirst.equals("Human")) {
      humanPlayer.turn();
    }
    else {
      machinePlayer.turn();
    }
  }

  /**
   * Set which player will go first
   * @param whoGoFirst The first player
   */
  protected void setWhoGoFirst(String whoGoFirst) {
    this.whoGoFirst = whoGoFirst;
  }

  /**
   * Updates the number of sticks left in the pile.
   * @param picksticks No. of sticks picked
   */
  protected void updateSticksLeft(int picksticks) {
    sticksLeft = sticksLeft - picksticks;
  }

  /**
   * Game Over?
   * @param player The player
   * @return true if the game is over.
   */
  protected boolean gameOver(Player player) {
    if (player == null) {
      return false;
    }
    else {
      return sticksLeft == 0;
    }
  }

  /**
   * Returns the current player
   * @return The current player
   */
  protected Player getCurrentPlayer() {
    return turn;
  }

  /**
   * Sets the current player
   * @param player The player
   */
  protected void setCurrentPlayer(Player player) {
    turn = player;
  }

  /**
   * Valid move?
   * @param numSticks The number of sticks picked
   * @return true iff there are enough sticks left and numSticks is between 1 and 3
   */

  protected boolean validMove(int numSticks) {
    return (
        numSticks >= 1 && numSticks <= 3 && sticksLeft - numSticks >= 0);
  }

  /**
   * Returns the number of sticks left in the pile
   * @return The number of sticks left in the pile
   */
  protected int getSticksLeft() {
    return sticksLeft;
  }

  /**
   * Displays a message
   * @param msg The message
   */
  public void displayMessage(String msg) {
    simpleNIMGUI.displayMessage(msg);
  }

  /**
   * Pick up Sticks
   * @param noOfSticks The number of sticks to be picked up
   * @param turn The current player
   */
  public void pickSticks(int noOfSticks, Player turn) {
    simpleNIMGUI.pickSticks(noOfSticks, turn);
  }
}
